// Generated by CoffeeScript 1.4.0
(function() {
  "use strict";

  describe('A window list', function() {
    var windows, wl;
    wl = windows = void 0;
    var createWindow = function(server, chan) {
      var win = new chat.Window(server, chan);
      if (chan) {
        win.setTarget(chan.toLowerCase());
      }
      win.conn = {
        name: server
      };
      windows.push(win);
      return win;
    };
    var joinMultipleServersAndChannels = function() {
      wl.add(createWindow('freenode'));
      wl.add(createWindow('freenode', '#bash'));
      wl.add(createWindow('freenode', '#zebra'));
      wl.add(createWindow('dalnet'));
      wl.add(createWindow('dalnet', '#bash'));
      return wl.add(createWindow('dalnet', '#zebra'));
    };
    beforeEach(function() {
      wl = new chat.WindowList();
      return windows = [];
    });
    it('returns undefined when getChannelWindow is called with no matching window', function() {
      return expect(wl.getChannelWindow(0)).toBeUndefined();
    });
    it('returns -1 when indexOf is called with no matching window', function() {
      expect(wl.indexOf(createWindow('freenode'))).toBe(-1);
      return expect(wl.indexOf(createWindow('freenode', '#bash'))).toBe(-1);
    });
    it('can have windows added', function() {
      wl.add(createWindow('freenode'));
      return wl.add(createWindow('freenode', '#bash'));
    });
    it('can have windows removed', function() {
      wl.add(createWindow('freenode'));
      wl.add(createWindow('freenode', '#bash'));
      wl.remove(windows[0]);
      return wl.remove(windows[1]);
    });
    it('throws an error when a channel window is added with no corresponding connection window', function() {
      var addChannelWindow = function() {
        return wl.add(createWindow('freenode', '#bash'));
      };
      return expect(addChannelWindow).toThrow();
    });
    it("returns undefined on getChannelWindow when it only has server windows", function() {
      wl.add(createWindow('freenode'));
      wl.add(createWindow('dalnet'));
      expect(wl.getChannelWindow(0)).toBeUndefined();
      return expect(wl.getChannelWindow(1)).toBeUndefined();
    });
    it("returns the nth window on get(n)", function() {
      joinMultipleServersAndChannels();
      var results = [];
      for (var i = 0; i <= 5; i++) {
        results.push(expect(wl.get(i)).toBe(windows[i]));
      }
      return results;
    });
    it("has a length property which is equal to the number of windows", function() {
      expect(wl.length).toBe(0);
      joinMultipleServersAndChannels();
      expect(wl.length).toBe(6);
      wl.remove(windows[0]);
      expect(wl.length).toBe(3);
      wl.add(windows[0]);
      return expect(wl.length).toBe(4);
    });
    it("returns undefined when get is called on a deleted window", function() {
      joinMultipleServersAndChannels();
      wl.remove(windows[0]);
      wl.remove(windows[4]);
      expect(wl.get('freenode')).toBeNull();
      return expect(wl.get('dalnet', '#bash')).toBeNull();
    });
    it("deletes all channel windows when their server window is deleted", function() {
      joinMultipleServersAndChannels();
      wl.remove(windows[0]);
      wl.remove(windows[4]);
      var indexes = [-1, -1, -1, 0, -1, 1];
      var results = [];
      for (var window = 0, len = indexes.length; window < len; window++) {
        var i = indexes[window];
        results.push(expect(wl.indexOf(windows[window])).toBe(i));
      }
      return results;
    });
    it("returns the Nth channel window on getChannelWindow(N)", function() {
      joinMultipleServersAndChannels();
      var indexes = [1, 2, 4, 5];
      var results = [];
      for (var i = 0, len = indexes.length; i < len; i++) {
        var window = indexes[i];
        results.push(expect(wl.getChannelWindow(i)).toBe(windows[window]));
      }
      return results;
    });
    it("returns the Nth server window on getServerWindow(N)", function() {
      joinMultipleServersAndChannels();
      return expect(wl.getServerWindow(1)).toBe(windows[3]);
    });
    it("returns the server window with the given name on getServerWindow(N)", function() {
      joinMultipleServersAndChannels();
      return expect(wl.getServerWindow(1)).toBe(windows[3]);
    });
    it("returns the window with the given server and channel on get(server, chan)", function() {
      joinMultipleServersAndChannels();
      expect(wl.get('freenode')).toBe(windows[0]);
      expect(wl.get('freenode', '#bash')).toBe(windows[1]);
      expect(wl.get('freenode', '#zebra')).toBe(windows[2]);
      expect(wl.get('dalnet')).toBe(windows[3]);
      expect(wl.get('dalnet', '#bash')).toBe(windows[4]);
      return expect(wl.get('dalnet', '#zebra')).toBe(windows[5]);
    });
    it("returns the index of the given window on indexOf(window)", function() {
      joinMultipleServersAndChannels();
      var results = [];
      for (var i = 0; i <= 5; i++) {
        results.push(expect(wl.indexOf(windows[i])).toBe(i));
      }
      return results;
    });
    it("sorts windows under the same server in alphabetical order by their channel", function() {
      wl.add(createWindow('freenode'));
      wl.add(createWindow('freenode', '#bash'));
      wl.add(createWindow('freenode', '#tcsh'));
      wl.add(createWindow('freenode', '#zebra'));
      wl.add(createWindow('freenode', '#Linux'));
      wl.add(createWindow('freenode', '#CIRC'));
      wl.add(createWindow('freenode', 'someuser'));
      wl.add(createWindow('freenode', '#GitHub'));
      wl.add(createWindow('freenode', '#foo'));
      wl.add(createWindow('freenode', '#bar'));
      wl.add(createWindow('freenode', '#Baz'));
      wl.add(createWindow('freenode', '#qqq'));
      wl.add(createWindow('freenode', '#freenode'));
      wl.add(createWindow('freenode', 'anotheruser'));
      var ordering = [0, 9, 1, 10, 5, 8, 12, 7, 4, 11, 2, 3, 13, 6];
      var results = [];
      for (var i = 0, len = ordering.length; i < len; i++) {
        var window = ordering[i];
        results.push(expect(wl.indexOf(windows[window])).toBe(i));
        results.push(expect(wl.get(i).target).toBe(windows[window].target));
      }
      return results;
    });
    it("sorts first by server, then by channel", function() {
      wl.add(createWindow('freenode'));
      wl.add(createWindow('dalnet'));
      wl.add(createWindow('dalnet', '#zebra'));
      wl.add(createWindow('freenode', '#zebra'));
      wl.add(createWindow('dalnet', '#bash'));
      wl.add(createWindow('freenode', '#bash'));
      var ordering = [5, 3, 4, 2];
      for (var i = 0, len = ordering.length; i < len; i++) {
        var window = ordering[i];
        expect(wl.getChannelWindow(i)).toBe(windows[window]);
      }
      ordering = [0, 5, 3, 1, 4, 2];
      var results = [];
      for (var i = 0, len = ordering.length; i < len; i++) {
        window = ordering[i];
        results.push(expect(wl.indexOf(windows[window])).toBe(i));
      }
      return results;
    });
    it("returns the window index on indexOf when it only has server windows", function() {
      wl.add(createWindow('freenode'));
      wl.add(createWindow('dalnet'));
      expect(wl.indexOf(windows[0])).toBe(0);
      return expect(wl.indexOf(windows[1])).toBe(1);
    });
    it("returns the window on get() when it only has server windows", function() {
      wl.add(createWindow('freenode'));
      wl.add(createWindow('dalnet'));
      expect(wl.get('freenode')).toBe(windows[0]);
      return expect(wl.get('dalnet')).toBe(windows[1]);
    });
    it("can return the index of a channel in the context of its server", function() {
      wl.add(createWindow('freenode'));
      wl.add(createWindow('dalnet'));
      wl.add(createWindow('dalnet', '#zebra'));
      wl.add(createWindow('freenode', '#zebra'));
      wl.add(createWindow('dalnet', '#bash'));
      wl.add(createWindow('freenode', '#bash'));
      expect(wl.localIndexOf(windows[2])).toBe(1);
      return expect(wl.localIndexOf(windows[5])).toBe(0);
    });
    return it("can return the server that corresponds with a given window", function() {
      joinMultipleServersAndChannels();
      expect(wl.getServerForWindow(windows[0])).toBe(windows[0]);
      expect(wl.getServerForWindow(windows[1])).toBe(windows[0]);
      expect(wl.getServerForWindow(windows[2])).toBe(windows[0]);
      expect(wl.getServerForWindow(windows[3])).toBe(windows[3]);
      expect(wl.getServerForWindow(windows[4])).toBe(windows[3]);
      return expect(wl.getServerForWindow(windows[5])).toBe(windows[3]);
    });
  });

}).call(this);
